do
local _ENV = _ENV
package.preload[ "json" ] = function( ... ) local arg = _G.arg;
--
-- json.lua
--
-- Copyright (c) 2020 rxi
--
-- Permission is hereby granted, free of charge, to any person obtaining a copy of
-- this software and associated documentation files (the "Software"), to deal in
-- the Software without restriction, including without limitation the rights to
-- use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
-- of the Software, and to permit persons to whom the Software is furnished to do
-- so, subject to the following conditions:
--
-- The above copyright notice and this permission notice shall be included in all
-- copies or substantial portions of the Software.
--
-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-- IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-- FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-- AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-- LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-- OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
-- SOFTWARE.
--

local json = { _version = "0.1.2" }

-------------------------------------------------------------------------------
-- Encode
-------------------------------------------------------------------------------

local encode

local escape_char_map = {
  [ "\\" ] = "\\",
  [ "\"" ] = "\"",
  [ "\b" ] = "b",
  [ "\f" ] = "f",
  [ "\n" ] = "n",
  [ "\r" ] = "r",
  [ "\t" ] = "t",
}

local escape_char_map_inv = { [ "/" ] = "/" }
for k, v in pairs(escape_char_map) do
  escape_char_map_inv[v] = k
end


local function escape_char(c)
  return "\\" .. (escape_char_map[c] or string.format("u%04x", c:byte()))
end


local function encode_nil(rope)
  rope[#rope + 1] = "null"
end


local function encode_table(rope, val, stack)
  -- Circular reference?
  if stack[val] then error("circular reference") end

  stack[val] = true
  if rawget(val, 1) ~= nil or next(val) == nil then
    -- Treat as array -- check keys are valid and it is not sparse
    local n = 0
    for k in pairs(val) do
      if type(k) ~= "number" then
        error("invalid table: mixed or invalid key types")
      end
      n = n + 1
    end
    if n ~= #val then
      error("invalid table: sparse array")
    end
    -- Encode
    rope[#rope + 1] = "["
    for i, v in ipairs(val) do
      if i > 1 then
        rope[#rope + 1] = ","
      end
      encode(rope, v, stack)
    end
    rope[#rope + 1] = "]"
  else
    -- Treat as an object
    rope[#rope + 1] = "{"
    local first = true
    for k, v in pairs(val) do
      if type(k) ~= "string" then
        error("invalid table: mixed or invalid key types")
      end
      if not first then
        rope[#rope + 1] = ","
      end
      encode(rope, k, stack)
      rope[#rope + 1] = ":"
      encode(rope, v, stack)
      first = false
    end
    rope[#rope + 1] = "}"
  end
  stack[val] = nil
end


local function encode_string(rope, val)
  rope[#rope + 1] = '"'
  rope[#rope + 1] = val:gsub('[%z\1-\31\\"]', escape_char)
  rope[#rope + 1] = '"'
end


local function encode_number(rope, val)
  -- Check for NaN, -inf and inf
  if val ~= val or val <= -math.huge or val >= math.huge then
    error("unexpected number value '" .. tostring(val) .. "'")
  end
  -- See www.cs.berkeley.edu/~wkahan/ieee754status/IEEE754.PDF
  -- 17 digits suffice to losslessly represent 64-bit IEEE754 floats
  rope[#rope + 1] = ("%.17g"):format(val)
end

local function encode_boolean(rope, val)
  rope[#rope + 1] = val and "true" or "false"
end

local type_func_map = {
  [ "nil"     ] = encode_nil,
  [ "table"   ] = encode_table,
  [ "string"  ] = encode_string,
  [ "number"  ] = encode_number,
  [ "boolean" ] = encode_boolean,
}


function encode(rope, val, stack)
  local t = type(val)
  local encoder = type_func_map[t]
  if encoder then
    return encoder(rope, val, stack)
  end
  error("unexpected type '" .. t .. "'")
end


function json.encode(val)
  local rope = {}
  encode(rope, val, {})
  return table.concat(rope)
end


-------------------------------------------------------------------------------
-- Decode
-------------------------------------------------------------------------------

local parse

local function create_set(...)
  local res = {}
  for i = 1, select("#", ...) do
    res[ select(i, ...) ] = true
  end
  return res
end

local space_chars   = create_set(" ", "\t", "\r", "\n")
local delim_chars   = create_set(" ", "\t", "\r", "\n", "]", "}", ",")
local escape_chars  = create_set("\\", "/", '"', "b", "f", "n", "r", "t", "u")
local literals      = create_set("true", "false", "null")

local literal_map = {
  [ "true"  ] = true,
  [ "false" ] = false,
  [ "null"  ] = nil,
}


local function next_char(str, idx, set, negate)
  for i = idx, #str do
    if set[str:sub(i, i)] ~= negate then
      return i
    end
  end
  return #str + 1
end


local function decode_error(str, idx, msg)
  local line_count = 1
  local col_count = 1
  for i = 1, idx - 1 do
    col_count = col_count + 1
    if str:sub(i, i) == "\n" then
      line_count = line_count + 1
      col_count = 1
    end
  end
  error( string.format("%s at line %d col %d", msg, line_count, col_count) )
end


local function codepoint_to_utf8(n)
  -- http://scripts.sil.org/cms/scripts/page.php?site_id=nrsi&id=iws-appendixa
  local f = math.floor
  if n <= 0x7f then
    return string.char(n)
  elseif n <= 0x7ff then
    return string.char(f(n / 64) + 192, n % 64 + 128)
  elseif n <= 0xffff then
    return string.char(f(n / 4096) + 224, f(n % 4096 / 64) + 128, n % 64 + 128)
  elseif n <= 0x10ffff then
    return string.char(f(n / 262144) + 240, f(n % 262144 / 4096) + 128,
                       f(n % 4096 / 64) + 128, n % 64 + 128)
  end
  error( string.format("invalid unicode codepoint '%x'", n) )
end


local function parse_unicode_escape(s)
  local n1 = tonumber( s:sub(1, 4),  16 )
  local n2 = tonumber( s:sub(7, 10), 16 )
   -- Surrogate pair?
  if n2 then
    return codepoint_to_utf8((n1 - 0xd800) * 0x400 + (n2 - 0xdc00) + 0x10000)
  else
    return codepoint_to_utf8(n1)
  end
end


local function parse_string(str, i)
  local res = {}
  local j = i + 1
  local k = j

  while j <= #str do
    local x = str:byte(j)

    if x < 32 then
      decode_error(str, j, "control character in string")

    elseif x == 92 then -- `\`: Escape
      res[#res + 1] = str:sub(k, j - 1)
      j = j + 1
      local c = str:sub(j, j)
      if c == "u" then
        local hex = str:match("^[dD][89aAbB]%x%x\\u%x%x%x%x", j + 1)
                 or str:match("^%x%x%x%x", j + 1)
                 or decode_error(str, j - 1, "invalid unicode escape in string")
        res[#res + 1] = parse_unicode_escape(hex)
        j = j + #hex
      else
        if not escape_chars[c] then
          decode_error(str, j - 1, "invalid escape char '" .. c .. "' in string")
        end
        res[#res + 1] = escape_char_map_inv[c]
      end
      k = j + 1

    elseif x == 34 then -- `"`: End of string
      res[#res + 1] = str:sub(k, j - 1)
      return table.concat(res), j + 1
    end

    j = j + 1
  end

  decode_error(str, i, "expected closing quote for string")
  return table.concat(res)
end


local function parse_number(str, i)
  local x = next_char(str, i, delim_chars)
  local s = str:sub(i, x - 1)
  local n = tonumber(s)
  if not n then
    decode_error(str, i, "invalid number '" .. s .. "'")
  end
  return n, x
end


local function parse_literal(str, i)
  local x = next_char(str, i, delim_chars)
  local word = str:sub(i, x - 1)
  if not literals[word] then
    decode_error(str, i, "invalid literal '" .. word .. "'")
  end
  return literal_map[word], x
end


local function parse_array(str, i)
  local res = {}
  local n = 1
  i = i + 1
  while 1 do
    local x
    i = next_char(str, i, space_chars, true)
    -- Empty / end of array?
    if str:sub(i, i) == "]" then
      i = i + 1
      break
    end
    -- Read token
    x, i = parse(str, i)
    res[n] = x
    n = n + 1
    -- Next token
    i = next_char(str, i, space_chars, true)
    local chr = str:sub(i, i)
    i = i + 1
    if chr == "]" then break end
    if chr ~= "," then decode_error(str, i, "expected ']' or ','") end
  end
  return res, i
end


local function parse_object(str, i)
  local res = {}
  i = i + 1
  while 1 do
    local key, val
    i = next_char(str, i, space_chars, true)
    -- Empty / end of object?
    if str:sub(i, i) == "}" then
      i = i + 1
      break
    end
    -- Read key
    if str:sub(i, i) ~= '"' then
      decode_error(str, i, "expected string for key")
    end
    key, i = parse(str, i)
    -- Read ':' delimiter
    i = next_char(str, i, space_chars, true)
    if str:sub(i, i) ~= ":" then
      decode_error(str, i, "expected ':' after key")
    end
    i = next_char(str, i + 1, space_chars, true)
    -- Read value
    val, i = parse(str, i)
    -- Set
    res[key] = val
    -- Next token
    i = next_char(str, i, space_chars, true)
    local chr = str:sub(i, i)
    i = i + 1
    if chr == "}" then break end
    if chr ~= "," then decode_error(str, i, "expected '}' or ','") end
  end
  return res, i
end


local char_func_map = {
  [ '"' ] = parse_string,
  [ "0" ] = parse_number,
  [ "1" ] = parse_number,
  [ "2" ] = parse_number,
  [ "3" ] = parse_number,
  [ "4" ] = parse_number,
  [ "5" ] = parse_number,
  [ "6" ] = parse_number,
  [ "7" ] = parse_number,
  [ "8" ] = parse_number,
  [ "9" ] = parse_number,
  [ "-" ] = parse_number,
  [ "t" ] = parse_literal,
  [ "f" ] = parse_literal,
  [ "n" ] = parse_literal,
  [ "[" ] = parse_array,
  [ "{" ] = parse_object,
}


parse = function(str, idx)
  local chr = str:sub(idx, idx)
  local f = char_func_map[chr]
  if f then
    return f(str, idx)
  end
  decode_error(str, idx, "unexpected character '" .. chr .. "'")
end


function json.decode(str)
  if type(str) ~= "string" then
    error("expected argument of type string, got " .. type(str))
  end
  local res, idx = parse(str, next_char(str, 1, space_chars, true))
  idx = next_char(str, idx, space_chars, true)
  if idx <= #str then
    decode_error(str, idx, "trailing garbage")
  end
  return res
end


return json
end
end

--|
--|    YouHelpMe
--| +=============+
--|
--| YouHelpMe (YHM) is a CS2D Lua script for showing hint and tips upon spawning.
--| Made just for fun and intended to be replacement for TipsSpawner.
--|
--| License can be found at EOF of this file.
--|

local json = require "json"

--> Misc. funcs <--

--- @param str string
--- @return string
local function append_0_rgb(str)
    local rgb = {}
    for s in str:gmatch "([^,]+)" do
        rgb[#rgb+1] = #s == 1 and "00"..s or
                      #s == 2 and "0"..s or s
    end
    return table.concat(rgb)
end

--- @param cmd string
--- @return fun(...: any)
local function p(cmd)
    return function(...)
        local p_cmd = cmd
        for i = 1,select('#',...) do
            local v = select(i,...)

            p_cmd = p_cmd..' "'..v..'"'
        end
        parse(p_cmd)
    end
end

--> API stuff <--

--- @class YHM
--- @field private _config_table table
--- @field _version string Version of YHM (in semver)
YHM = { _version = "0.2", _config_table = {
    messages = { T = {}, CT = {} },
    colorMessage = {}
}}

local f = io.open("sys/yhm-config.json",'r')

if f then
    YHM._config_table = json.decode(f:read "*a")
    f:close()
end

local config_colors = YHM._config_table.colorMessage
config_colors["defaultColor"] = config_colors["defaultColor"] or "255,255,255"
config_colors["tips"] = config_colors["tips"] or "255,145,145"
config_colors["hint"] = config_colors["hint"] or "255,239,145"

--[[
Return an iterator function which iterate team messages. Iterator function return the message of the team.
]]
--- @param team string
--- @return fun(): string
--- @see YHM.get_all_messages
function YHM:get_messages_from_team(team)
    local i = 0
    local n = #self._config_table.messages[team]
    --- @return string
    return function()
        i = i + 1
        --- @diagnostic disable-next-line:missing-return
        if i <= n then return self._config_table.messages[team][i] end
    end
end

--[[
Return an iterator function which iterate all messages in `messages`, even if it's `T` or `CT`. Iterator function return name of the team and the message.
]]
--- @return fun(): string, string
--- @see YHM.get_messages_from_team
function YHM:get_all_messages()
    local tmp = {}
    for t,v in pairs(self._config_table.messages) do
        for _,m in ipairs(v) do
            tmp[#tmp+1] = {t,m}
        end
    end

    local i = 0
    local n = #tmp
    --- @return string
    --- @return string
    return function()
        i = i + 1
        --- @diagnostic disable-next-line:missing-return
        if i <= n then return tmp[i][1], tmp[i][2] end
    end
end

--[[
Return RGB color based on prefix. If passed boolean `true` on append_zero, append zero on each comma-separated value and return RGB color without comma separator.
]]
--- @param prefix string
--- @param append_zero? boolean
--- @return string
--- @see YHM.get_all_colors
function YHM:get_color_by_prefix(prefix, append_zero)
    return not append_zero and
        self._config_table.colorMessage[prefix] or
        append_0_rgb(self._config_table.colorMessage[prefix])
end

--[[
Return an iterator function which iterate over all `colorMessage` property. Iterator function returns prefix of the color and the RGB color value. If passed boolean `true` on append_zero, second return of iterator function appends zero on each comma-separated value and removes comma separator.
]]
--- @param append_zero? boolean
--- @return fun(): string, string
--- @see YHM.get_color_by_prefix
function YHM:get_all_colors(append_zero)
    local tmp = {}
    for i,v in pairs(self._config_table.colorMessage) do
        tmp[#tmp+1] = {i, not append_zero and v or append_0_rgb(v)}
    end

    local i = 0
    local n = #tmp
    --- @return string
    --- @return string
    return function()
        i = i + 1
        --- @diagnostic disable-next-line:missing-return
        if i <= n then return tmp[i][1], tmp[i][2] end
    end
end

--[[
Get the current loaded config if `sys/yhm-config.json` found, if not return nil or return new loaded config by `YHM:overwrite_config`.
]]
--- @param tbl? any
--- @see YHM.get_config
function YHM:overwrite_config(tbl)
    self._config_table = tbl or {}
end

--[[
Load a new config. If config from `sys/yhm-config.json` found, this function overwrite the loaded config (config file does not overwriten by this function).
]]
--- @return table|nil
--- @see YHM.overwrite_config
function YHM:get_config()
    return self._config_table
end

--> Main section <--

local team = {"T","CT"}

function YHM_clear_hudtxt2(id)
    p"hudtxt2" (id,10,"",0,0)
end

addhook("spawn","YHM_spawn_hook")
function YHM_spawn_hook(id)
    local team_id = player(id,"team")
    local msg_config = YHM._config_table.messages[team[team_id]]

    if msg_config then
        local n = #msg_config
        ---@type string
        local yhm_msg = msg_config[math.random(1,n)]
        local prefix_msg = (yhm_msg:match "^([a-z0-9]+):" or ""):lower()

        p"hudtxt2" (id,10,
                   "\169"..
                   append_0_rgb(config_colors[prefix_msg] or config_colors["defaultColor"])
                    ..yhm_msg,
                   player(id,"screenw")/2,(player(id,"screenh")/2) - 180,
                   1,1,18)
        timer(3000,"YHM_clear_hudtxt2",tostring(id))
    end
end

--| ==============================================================================
--| The MIT License (MIT)
--| 
--| Copyright (c) 2023 UrNightmaree
--| 
--| Permission is hereby granted, free of charge, to any person obtaining a copy
--| of this software and associated documentation files (the "Software"), to deal
--| in the Software without restriction, including without limitation the rights
--| to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
--| copies of the Software, and to permit persons to whom the Software is
--| furnished to do so, subject to the following conditions:
--| 
--| The above copyright notice and this permission notice shall be included in all
--| copies or substantial portions of the Software.
--| 
--| THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
--| IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
--| FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
--| AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
--| LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
--| OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
--| SOFTWARE.
--| ==============================================================================
